// %flair:license{
// This file is part of the Flair framework distributed under the
// CECILL-C License, Version 1.0.
// %flair:license}
//  created:    2016/07/05
//  filename:   main.cpp
//
//  author:     Thomas Fuhrmann
//              Copyright Heudiasyc UMR UTC/CNRS 7253
//
//  version:    $Id: $
//
//  purpose:    main for the mavlink controller
//
//
/*********************************************************************/

#include "FrameworkManager.h"
#include "Mavlink_test.h"
#include <stdio.h>
#include <tclap/CmdLine.h>

using namespace TCLAP;
using namespace std;
using namespace flair::core;
using namespace flair::sensor;

string receiverAddress;
int receiverPort;
int port;
string xml_file;

void parseOptions(int argc, char **argv);

int main(int argc, char *argv[]) {
  parseOptions(argc, argv);

  FrameworkManager *manager;
  Mavlink *mavlink_handler;

  manager = new FrameworkManager("mavlink");
  manager->SetupConnection("127.0.0.1", port);
  manager->SetupUserInterface(xml_file);

  mavlink_handler = new Mavlink(manager, "mavlink", receiverAddress,
                                   receiverPort);

  if (!manager->ErrorOccured()) {
    mavlink_handler->Start();
    mavlink_handler->Join();
  }

  delete manager;
}

void parseOptions(int argc, char **argv) {
  try {
    CmdLine cmd("Command description message", ' ', "0.1");

    ValueArg<string> addressArg("a", "address",
                                "data receiver address (ex: uav)", true,
                                "127.0.0.1:20000", "string");
    cmd.add(addressArg);

    ValueArg<int> portArg("p", "port",
                          "local port used to connect to the ground station",
                          false, 9000, "int");
    cmd.add(portArg);

    ValueArg<string> xmlArg("x", "xml", "xml file", true, "./settings.xml",
                            "string");
    cmd.add(xmlArg);

    cmd.parse(argc, argv);

    // Get the value parsed by each arg.
    string receiverAddressWithPort = addressArg.getValue();
    int semiColonPosition = receiverAddressWithPort.find(":");
    receiverAddress = receiverAddressWithPort.substr(0, semiColonPosition);
    receiverPort =
        atoi(receiverAddressWithPort.substr(semiColonPosition + 1).c_str());
    port = portArg.getValue();
    xml_file = xmlArg.getValue();

  } catch (ArgException &e) { // catch any exceptions
    cerr << "error: " << e.error() << " for arg " << e.argId() << endl;
  }
}
